# Copyright (C) 2014 by Alex Brandt <alunduil@alunduil.com>
#
# crumbs is freely distributable under the terms of an MIT-style license.
# See COPYING or http://www.opensource.org/licenses/mit-license.php.

NAME = 'crumbs'
VERSION = '2.0.1'
DESCRIPTION = 'Generalized all-in-one parameters module.'
AUTHOR = 'Alex Brandt'
AUTHOR_EMAIL = 'alunduil@alunduil.com'
URL = 'https://github.com/alunduil/crumbs'
LICENSE = 'MIT'

COPY_YEAR = '2014'
